/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.fukurou.xml;

/**
 * JSP/XMLファイルを読み取って、構築される最上位の OGDocument オブジェクト を定義します。
 *
 * JspSaxParser での read/write で構築される トップレベルの OGNode になります。
 *
 * @og.rev 5.1.8.0 (2010/07/01) 新規作成
 *
 * @version  5.0
 * @author   Kazuhiko Hasegawa
 * @since    JDK6.0,
 */
public class OGDocument extends OGNode {

	/**
	 * デフォルトコンストラクター
	 *
	 * NodeType を Document に設定し、デフォルトの OGNode を構築します。
	 *
	 */
	public OGDocument() {
		super();
		setNodeType( OGNodeType.Document );
	}

	/**
	 * 最上位の エレメントを返します。
	 *
	 * ノードが、まだ登録されていない、または、エレメントノードでない場合は、null が返されます。
	 *
	 * @return OGElement
	 */
	public OGElement getTopElement() {
		if( nodeSize() > 0 ) {
			OGNode node = getNode( 0 );
			if( node.getNodeType() == OGNodeType.Element ) {
				return (OGElement)node;
			}
		}
		return null ;
	}
}
